package Source::Repository::RPM::ALTLinuxSrcList;

use strict;
use warnings;
use RPM::Vercmp;

use Source::Shared::Utils::Downloader qw/named_cache_download/;
use Source::Repository::RPM;
our @ISA=qw/Source::Repository::RPM/;
our $verbose=0;

# Note: does not need src.rpms to be actually present

sub create_repo_from_dir {
    my ($class, $opt, $dir)=@_;
    $dir .= '/' if $dir !~ m!/$!;
    my $altsrclist=$dir.'../list/src.list';
    if (-e $altsrclist) {
	my ($names, $name2evr, $name2suffix)=&__load_altsrclist($altsrclist);
	return $class->new(
	    -fileprefix => $dir,
	    -created_from => $dir,
	    %$opt,
	    -names => $names,
	    -name2evr => $name2evr,
	    -name2suffix => $name2suffix,
	    );
    } else {
	return $class->SUPER::create_repo_from_dir($opt, $dir);
    }
}

sub create_repo_from_file {
    my ($class, $opt, $altsrclist)=@_;
    if ($altsrclist!~ m!/list/src.list$!) {
	return $class->SUPER::create_repo_from_file($opt, $altsrclist);
    } else {
	my $dir=$altsrclist;
	$dir=~s!/list/src.list$!/SRPMS/!;
	my ($names, $name2evr, $name2suffix)=&__load_altsrclist($altsrclist);
	return $class->new(
	    -fileprefix => $dir,
	    -created_from => $altsrclist,
	    %$opt,
	    -names => $names,
	    -name2evr => $name2evr,
	    -name2suffix => $name2suffix,
	    );
    }
}

sub create_repo_from_url {
    my ($class, $opt, $altsrclist_url)=@_;
    if ($altsrclist_url!~ m!/list/src.list$!) {
	print STDERR "DEBUG: not a src.list: $altsrclist_url\n" if $verbose >2;
	return $class->SUPER::create_repo_from_url($opt, $altsrclist_url);
    } else {
	my $dir=$altsrclist_url;
	$dir=~s!/list/src.list$!/SRPMS/!;
	my ($names, $name2evr, $name2suffix)=&__load_altsrclist_url($altsrclist_url,$opt->{-cachedir});
	die "bad URL $altsrclist_url: empty list" if ! %$name2suffix;
	return $class->new(
	    -urlprefix => $dir,
	    -created_from => $altsrclist_url,
	    %$opt,
	    -names => $names,
	    -name2evr => $name2evr,
	    -name2suffix => $name2suffix,
	    );
    }
}

sub name2pkg {
    my ($self,$name)=@_;
    my $name2suffix=$self->{-name2suffix};
    return $self->SUPER::name2pkg($name) if ! $name2suffix;
    my $suffix=$name2suffix->{$name};
    return unless $suffix;
    my $evr=$self->{-name2evr}->{$name};
    if ($evr=~m!^(?:(\d+):)?([^-]+)-(.*)$!) {
	my $prefix=$self->{-fileprefix};
	my $urlprefix=$self->{-urlprefix};
	if ($urlprefix) {
	    return $self->get_package_class->new(
		-name => $name,
		-epoch=> $1,
		-version => $2,
		-release => $3,
		-cachedir => $self->{-cachedir},
		-url => $urlprefix.$suffix,
		);
	} else {
	    return $self->get_package_class->new(
		-name => $name,
		-epoch=> $1,
		-version => $2,
		-release => $3,
		-file => $prefix.$suffix,
		);
	}
    } else {
	die 'oops! ',__PACKAGE__, ': bad evr: ',$evr;
    }
}


sub __load_altsrclist {
    my ($list)=@_;
    open (my $fn, '<', $list) or die "can't open file $list\n";
    my @ret=&__load_altsrclist_fn($fn);
    close($fn);
    return @ret;
}

sub __load_altsrclist_url {
    my ($url,$cachedir)=@_;
    if ($url =~ s!^rsync://!!) {
	#my $altsrclist=&__get_rsync_url_cache($cachedir,'altsrclist',$url);
	my $altsrclist=named_cache_download($url,'altsrclist',$cachedir);
	return &__load_altsrclist($altsrclist);
    } else {
	my $cmd="wget -q -c -O- $url |";
	open (my $fn, $cmd) or die "can't open $cmd\n";
	my @ret=&__load_altsrclist_fn($fn);
	close($fn);
	return @ret;
    }
}

sub __load_altsrclist_fn {
    my ($fn)=@_;
    my @all_names;
    my %name2evr;
    my %name2suffix;
    while (my $line=<$fn>) {
	chomp $line;
	my @list = split(/\s+/,$line);
	next if @list!=3;
	my ($name,$evr,$suffix)=@list;
	my $oldevr=$name2evr{$name};
	if (! defined($oldevr)) {
	    push @all_names, $name;
	    $name2evr{$name}=$evr;
	    $name2suffix{$name}=$suffix;
	} elsif (RPM::Vercmp::evrcmp($oldevr, $evr)<=0) {
	    $name2evr{$name}=$evr;
	    $name2suffix{$name}=$suffix;
	}
    }
    return \@all_names, \%name2evr, \%name2suffix;
}

1;

__END__
use File::Basename;
use File::Path qw(make_path remove_tree);
sub __get_rsync_url_cache {
    my ($cachedir,$cachename,$url)=@_;
    my $CACHE_ROOT=$cachedir.'/'.$cachename.'/';
    my $rsync_url=$url;
    $rsync_url =~ s!^rsync://!! if $rsync_url =~ /::/;
    my $indir=$CACHE_ROOT.dirname($rsync_url).'/';
    make_path($indir);
    my @cmd=('rsync', '-avp', $rsync_url, $indir);
    system(@cmd)==0 or die "Error: ",join(' ',@cmd), " failed.";
    return $CACHE_ROOT.$url;
}
